/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Torisugari.
 *
 * Contributor(s):
 *  Torisugari (Original author)
 *  Nickolay Ponomarev <asqueella@gmail.com>
 *  Malte Kraus <firefox@maltekraus.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* upload.js
 *
 * (almost) Generic uploader + UI.
 *
 * Based on Infolister code which is based on Torisugari's BookmarksFTP code.
 *
 */

// XXX incorrect path - (no "/")users/mozilla/testing/123.html shows no message and does not upload

const Ci = Components.interfaces;
const Cc = Components.classes;
const Cr = Components.results;


const MAGIC_NUMBER = 0x804b0000;
const netErrorStrings = {
  "1": "unexpected",
  "2": "usercancel",
  "13": "refused",
  "14": "netTimeout",
  "16": "netOffline",
  "21": "ftplogin",
  "22": "ftpcwd",
  "23": "ftppasv",
  "24": "ftppwd",
  "25": "ftplist",
  "30": "unknown"
}; // others listed in nsNetError.h
var gUploadService = {
  // process aJob - uploads contents of aJob.streamToUpload to aJob.targetURI
  sendData: function(aJob) {
    try {
      var uri = aJob.targetURI;
      var stream = aJob.streamToUpload;
      if(!stream || !uri) {
        flagedExit(-1);
        return;
      }
      LOG("uploadService.sendData: uploading " + stream + " to " + uriToText(uri) + ".");

      var iosvc = Cc["@mozilla.org/network/io-service;1"]
                    .getService(Ci.nsIIOService);
      var channel = this.channel = iosvc.newChannelFromURI(uri);
      channel.QueryInterface(Ci.nsIUploadChannel);

      gOldNotificationCallbacks = channel.notificationCallbacks;
      channel.notificationCallbacks = gNotificationCallbacks;

      LOG("uploadService.sendData: calling " + channel + ".setUploadStream()");
      channel.setUploadStream(stream, aJob.contentType, -1);

      LOG("calling asyncOpen()...");
      setStatusText("send", null);
      channel.asyncOpen(gStreamListener, window);
    } catch(e) {
      LOG("uri=" + uriToText(uri));
      Components.reportError(e);
      flagedExit(-2);
    }
  },

  cancel: function() {
    if(this.channel)
      this.channel.cancel(MAGIC_NUMBER + 2);
    gJob.onNotUploaded();
  }
};

function uriToText(uri) {
  var urispec = null;
  if(uri instanceof Ci.nsIURI) {
    uri.userPass = "";
    var textToUri = Cc["@mozilla.org/intl/texttosuburi;1"].getService(Ci.nsITextToSubURI);
    urispec = textToUri.unEscapeURIForUI(uri.originCharset, uri.spec);
  }
  return urispec;
}

var gStreamListener = {
  // nsIStreamListener
  onDataAvailable: function(aChannel, aCtxt, aInStr, aSourceOffset, aCount) {},
  // nsIRequestObserver
  onStartRequest: function(aChannel, aCtxt) {},
  onStopRequest: function(aChannel, aCtxt, aErrCode) {
    LOG("onStopRequest(" + aErrCode + ")");
    if(aCtxt instanceof Ci.nsIDOMWindowInternal)
      aCtxt.flagedExit(aErrCode);
  }
};

var gOldNotificationCallbacks = null;
var gNotificationCallbacks = {
  getInterface: function(aIID) {
    try {
      return this.QueryInterface(aIID)
    } catch(e) {}
    if(gOldNotificationCallbacks instanceof Ci.nsIInterfaceRequestor)
      return gOldNotificationCallbacks.getInterface(aIID);
    return null;
  },
  prompt: function() { throw Cr.NS_ERROR_NOT_IMPLEMENTED; },
  promptPassword: function() { throw Cr.NS_ERROR_NOT_IMPLEMENTED; },
  promptUsernameAndPassword: function(title, text, realm, savePw, user, password) {
    var login = gJob.getUsernameAndPassword();
    if(!login) return false;
    user.value = login.user;
    password.value = login.password;
    return true;
  },
  QueryInterface: function(aIID) {
    if(aIID.equals(Ci.nsIAuthPrompt) || aIID.equals(Ci.nsISupports) ||
       aIID.equals(Ci.nsIInterfaceRequestor))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE;
  }
};


/*******  Dialog interface  ***************************************************/
var gLifeTime = 2;
var gStringBundle;
var gStatusBox;
var gJob;
var gSilent; // whether to close on error

//          "@mozilla.doslash.org/infolister/uploadjob;1", aFlag == SILENT_FLAG);
function onLoad()
{
  gStringBundle = document.getElementById("strbundle");
  gStatusBox = document.getElementById("status");

  gJob = window.arguments[0];
  gSilent = (1 in window.arguments) ? window.arguments[1] : false;

  if(gJob) {
    // gJob = Cc[gJob].CreateInstance().wrappedJSObject; // xxx
    setStatusText("init", null);
    setTimeout(startUpload, 0);
  } else {
    setStatusText("unexpected", -3);
    window.getAttention();
  }
}

function startUpload() {
  gUploadService.sendData(gJob);
}

// updates status text and sets close-window timeout if 
// aCode does not indicate that an error has oCcured
function flagedExit(aCode)
{
  setStatusText("", aCode);
  const DELAY = 2000;

  var elem = document.documentElement;
  elem.getButton("cancel").hidden = true;
  elem.getButton("accept").hidden = false;

  if(gSilent) {                 // silent mode
    if(aCode == 0)              // suCcess - close immediately
      setTimeout(onSuCcessfulUpload, 0);
    else                        // failure - close after delay
      setTimeout(onUnsuccessfulUpload, DELAY);
  } else {                      // normal mode
    if(aCode == 0) {            // suCcess - close after delay
      setTimeout(onSuCcessfulUpload, DELAY);
    } else {                    // failure - don't close
      window.getAttention();
      Cc["@mozilla.org/sound;1"].createInstance(Ci.nsISound).beep();
    }
  }
}

function onSuCcessfulUpload() {
  gJob.onUploaded();
  window.close();
}
function onUnsuccessfulUpload() {
  gJob.onNotUploaded();
  window.close();
}

function getErrorStr(aCode)
{
  if(aCode == 0) return "done";
  if(aCode < MAGIC_NUMBER) return "notneterror";
  aCode %= MAGIC_NUMBER;
  LOG(aCode);
  if(aCode in netErrorStrings)
    return netErrorStrings[aCode];
  return "unexpected";
}

function getLocalizedString(aStr)
{
  try {
    return gStringBundle.getString(aStr);
  } catch(e) {}
  return aStr;
}

// aMsg is either a blank string or one of strings from neterrors.properties,
// aErrCode is either undefined or an error code.
// At least one of params must be defined.
function setStatusText(aMsg, aErrCode)
{
  var status;
  if(aMsg) {
    status = getLocalizedString(aMsg);
  } else {
    aMsg = getErrorStr(aErrCode);
    status = getLocalizedString(aMsg);
  }

  if(aMsg == "notneterror" || aMsg == "unexpected")
    status += " (" + aErrCode + ")";

  gStatusBox.value = status;
}

function LOG(aStr) {
  //dump(aStr);

  var consoleService = Cc["@mozilla.org/consoleservice;1"]
                         .getService(Ci.nsIConsoleService);
  consoleService.logStringMessage(aStr);
}